/**
 * Creates a dialog to edit a region if used on a table or regions
 *
 * ### Examples
 *
 * JavaScript:
 *
 *
 *    $('.ep-region-table').shopUiRegiondialog();
 *
 * HTML:
 *   <table class="ep-region-table">
 *     <tr data-regions='{"objectId": #ID, "alias": "#NameOrAlias", "selectedCountries": [#JOIN(", ", #Countries)"#CountryID"#ENDJOIN]}'>
 *       <td class="Checkbox">
 *         <input name="ListObjectID" type="checkbox" value="#ID" class="ep-js" data-js=".uiInput()" />
 *       </td>
 *       <td>
 *         <a class="ep-regions-link">
 *           <span class="ep-regions-alias">#NameOrAlias</span>
 *         </a>
 *         <span class="fa fa-pencil MarginLeftTiny"></span>
 *       </td>
 *     </tr>
 *   <table
 *
 * @class jQuery.ui.shopUiRegiondialog
 * @extends jQuery.widget
 *
 * @uses ep
 * @uses de_epages
 * @since 7.12.0
 */

/*
 * @copyright       © Copyright 2006-2017, epages GmbH, All Rights Reserved.
 *
 * @module          de_epages.shop.ui.regiondialog
 *
 * @revision        $Revision: 1.20 $
 */
/*globals define*/
/*jslint nomen: true*/

define('de_epages/shop/ui/regiondialog', [
    'jquery',
    'ep',
    'de_epages',
    '$dict!./regiondialog',

    'jquery/metaparse',
    'jquery/tmpl',
    'jquery/ui/widget',
    'ep/ajax',
    'ep/ui/dialog',
    'ep/ui/validate',
    '$ready!'
], function ($, ep, de_epages, dict) {
    /*
     * @dictionary              ep.dict
     *
     * @translation             {Cancel}
     *                          {Save}
     *                          {UnknownError}
     *
     * @dictionary              de_epages.shop.dictionary
     *
     * @translation             {AliasMissing}
     *                          {NoCountrySelected}
     *
     * @dictionary              de_epages.shop.ui.regiondialog
     *
     * @translation             {EditRegion}
     *                          {DoubleRegion}
     */

    var translations = {
            cancel: dict.translate('Cancel'),
            editRegion: dict.translate('EditRegion'),
            save: dict.translate('Save'),
            NoCountrySelected: dict.translate('NoCountrySelected'),
            AliasMissing: dict.translate('AliasMissing'),
            OBJECT_EXISTS: dict.translate('DoubleRegion'),
            defaultError: dict.translate('UnknownError')
        };

    $.widget('ui.shopUiRegiondialog', {
        templateElements: {},

        options: {
            dlgTmpl: $('#ep-add-new-item-tmpl')
        },

        _init: function() {
            var self = this,
                o = self.options,
                elem = self.element;

            self.tmpl = o.dlgTmpl.tmpl();

            self.tmpl.find('.DialogError').removeClass('DialogError');
            self.tmpl.find('.ep-notification-danger').remove();

            self.templateElements.checkboxesHeadline = self.tmpl.find('.ep-regions-dialogcountrylistheadline');
            self.templateElements.checkboxes = self.tmpl.find('input[type=checkbox]');
            self.templateElements.regionId = self.tmpl.find('ep-regions-objectid');
            self.templateElements.alias = self.tmpl.find('.ep-regions-newalias');

            self.rows = self.element.find('tr');

            self.rows.find('.ep-regions-link').on('click', {self: self}, self._handleClick);
        },

        _openDialog: function () {
            var self = this;

            if (self.dialog) {
                self.dialog.uiDialog('open');

            } else {
                self.dialog = ep(self.tmpl).uiDialog({
                    modal: true,
                    autoOpen: true,
                    width: 500,
                    title: translations.editRegion,
                    close: function() {
                        $(this).off('keydown.enterAddNew');
                        if (self.tooltip) {
                            self.tooltip.uiTooltip('hide');
                        }
                        if (self.templateElements.alias.data('uiUiValidate') && self.templateElements.alias.data('uiUiValidate').tooltip) {
                            self.templateElements.alias.data('uiUiValidate').tooltip.uiTooltip('hide');
                        }
                    },
                    open: function() {
                        /* handle enter button */
                        $(this).on('keydown.enterAddNew', function(evt) {
                            if (evt.keyCode === 13) {
                                evt.preventDefault();
                                evt.stopPropagation();

                                self._handleSubmit();
                            }
                        });

                        self.templateElements.checkboxes.each(function(index, elem) {
                            this.checked = (self.dialogSettings.selectedCountries.indexOf(this.value) !== -1);
                        });
                        self.templateElements.regionId.val(self.dialogSettings.objectId);
                        self.templateElements.alias.val(self.dialogSettings.alias);
                    },
                    buttons: [{
                        cssClass: 'ep-button ep-button-add',
                        text: translations.save,
                        click: function() {
                            self._handleSubmit();
                        }
                    },
                    {
                        cssClass: 'ep-button ep-button-cancel',
                        text: translations.cancel,
                        click: function() {
                            $(this).dialog('close');
                        }
                    }]
                });

                self.dialog.find('*').metaparse();
            }
        },

        _handleSubmit: function() {
            var self = this,
                /* store old settings within the scope of the ajax call to revert in case of error */
                originalRow = self.currentRow,
                originalSettings = jQuery.extend(true, {}, self.dialogSettings),
                originalCountries = self.currentRow.find('.ep-regions-selectedcountries').html(),
                newCountryList = [];

            self.dialogSettings.alias = self.templateElements.alias.val().replace(/"/g, '\"');
            self.dialogSettings.selectedCountries = [];

            self.templateElements.checkboxes.each(function(index, elem) {
                if (this.checked) {
                    self.dialogSettings.selectedCountries.push(this.value);
                    newCountryList.push($(this).data('countryname'));
                }
            });

            if (!self.dialogSettings.selectedCountries.length || !self.templateElements.alias.val().length) {
                if (!self.templateElements.alias.val().length) {
                    self.templateElements.alias.data('uiUiValidate').tooltip.uiTooltip('show');

                } else if (!self.dialogSettings.selectedCountries.length) {
                    self._handleTooltip({
                        message: translations['NoCountrySelected'],
                        interactiveElement: self.templateElements.checkboxesHeadline,
                        context: self.templateElements.checkboxesHeadline,
                        orientation: 'bottom middle'
                    });
                }

                self.dialogSettings = originalSettings;
                return;
            }

            self.currentRow.data('regions', self.dialogSettings);
            self.currentRow.find('.ep-regions-alias').empty().text(self.dialogSettings.alias);
            self.currentRow.find('.ep-regions-selectedcountries')
                .empty()
                .text(newCountryList.sort().slice(0, 5).join(', ') + ((newCountryList.length > 5) ? ', ...' : ''))
                .prop('title', newCountryList.sort().join(', '));

            ep.ajax({
                dataType: 'json',
                data: {
                    ViewAction: 'JSONViewResponse',
                    ObjectID: self.dialogSettings.objectId,
                    ChangeAction: 'UnityEditRegion',
                    Alias: self.dialogSettings.alias,
                    CountryIDs: JSON.stringify(self.dialogSettings.selectedCountries)
                }
            })
            .done(function(response){
                if (!response.success) {
                    if (response.error) {
                        self._handleError({
                            reason: response.error[0] ? response.error[0].reason : 'defaultError',
                            originalRow: originalRow,
                            originalSettings: originalSettings,
                            originalCountries: originalCountries
                        });
                    }
                }
            })
            .fail(function(jqXHR, textStatus, errorThrown) {
                self._handleError({
                    reason: ($.isArray(errorThrown) && errorThrown[0].Reason) ? errorThrown[0].Reason : 'defaultError',
                    originalRow: originalRow,
                    originalSettings: originalSettings,
                    originalCountries: originalCountries
                });
            });

            self.dialogSettings = {};
            self.currentRow = undefined;

            self.dialog.uiDialog('close');
        },

        _handleError: function (opts) {
            var self = this;

            self._handleTooltip({
                message: (translations[opts.reason] !== undefined) ? translations[opts.reason] : translations['defaultError'],
                interactiveElement: opts.originalRow,
                context: opts.originalRow.find('.ep-regions-link'),
                orientation: 'bottom middle'
            });

            opts.originalRow.data('regions', opts.originalSettings);
            opts.originalRow.find('.ep-regions-alias').empty().text(opts.originalSettings.alias);
            opts.originalRow.find('.ep-regions-selectedcountries').empty().text(opts.originalCountries);
        },

        // creates a tooltip
        _initTooltip: function (options) {
            var context = options.context;

            this.tooltip = $('<div>')
                .uiTooltip({
                    interactive: true,
                    event: 'hover',
                    context: context,
                    orientation: options.orientation,
                    oldhandling: true,
                    additionalClass: 'ep-formfield-tooltip ep-uiValidate-info'
                });
        },

        // handle tooltips
        _handleTooltip: function (options) {
            var self = this,
                context = options.context,
                orientation = options.orientation ? options.orientation : 'bottom right';

            // create a tooltip if necessary
            if (self.tooltip === undefined) {
                self._initTooltip({
                    context: context,
                    orientation: orientation
                });
            }

            // update tooltip options
            self.tooltip.uiTooltip('option', 'context', context);
            self.tooltip.html(options.message);
            self.tooltip.uiTooltip('show');

            options.interactiveElement.one('mouseenter', function () {
                self.tooltip.uiTooltip('hide');
            });
        },

        _handleClick: function(evt) {
            var self = evt.data.self;

            self.currentRow = $(this).closest('tr');

            self.dialogSettings = jQuery.extend(true, {}, self.currentRow.data('regions'));

            self._openDialog();
        },

        destroy: function () {
            var self = this;

            self._superApply(arguments);
            self.element.empty();
        }

    });

    return de_epages;
});
