(function () {
    'use strict';

    // called in scribe-loader
    var scribePluginColorpicker = function (toolbarNode, options) {
       var colorPicker = toolbarNode.querySelector(options.selector);

        return function (scribe) {
            // if no color picker button exists
            if (!colorPicker) {
                return;
            }

            // create command for the color picker button
            var nodeName = 'SPAN',
                stylePattern = /^color:[^;]+;$/,
                epColorPicker,

                /**
                 * extends selection to whole span. Returns span node or undefined
                 *
                 * @param {selection} object - reference to a wrapped scribe editor selection object
                 */
                selectSpanContent = function(selection) {
                    var node,
                        range;

                    // if nothing is selected
                    if (typeof selection.range === 'undefined' || selection.range.collapsed) {
                        node = selection.getContaining(function(testNode) {
                            return (testNode.nodeName === nodeName && stylePattern.test(testNode.style.cssText));
                        });

                        // if a span node was found
                        if (node) {
                            range = document.createRange();
                            range.selectNode(node);
                            selection.selection.removeAllRanges();
                            selection.selection.addRange(range);
                        }
                    }

                    return node;
                },

                /**
                 * replace a selection/range with a html fragment
                 *
                 * @param {scribe} object - reference to scribe editor
                 * @param {content} string - html string with the new content
                 * @param {selectContent} boolean - true if content should be selected after insertion
                 */
                replaceSelectionWithHtml = function(scribe, content, selectContent) {
                    var scribeSelection,
                        selection,
                        range,
                        fragment,
                        firstInsertedNode,
                        lastInsertedNode,
                        fragmentRoot,
                        child;

                    // get selection via scribe instance
                    scribeSelection = new scribe.api.Selection();
                    selection = scribeSelection.selection;

                    // check if Selection object contains at least one range
                    if (selection.getRangeAt && selection.rangeCount) {
                        // get the first range and remove it
                        range = selection.getRangeAt(0);
                        range.deleteContents();

                        // check if browser supports range.createContextualFragment method
                        if (range.createContextualFragment) {
                            // create html fragment from content
                            fragment = range.createContextualFragment(content);

                        // workaround for non supporting browsers
                        } else {
                            // create html fragment from content
                            fragmentRoot = document.createElement("div");
                            fragmentRoot.innerHTML = content;
                            fragment = document.createDocumentFragment();

                            // get first child of fragment root
                            child = fragmentRoot.firstChild;

                            while (child) {
                                // append child to fragement (means that child gets removed from fragmentRoot)
                                fragment.appendChild(child);
                                child = fragmentRoot.firstChild;
                            }
                        }

                        // get nodes for selection (=> selectContent === true)
                        firstInsertedNode = fragment.firstChild;
                        lastInsertedNode = fragment.lastChild;

                        // insert new fragment as range
                        range.insertNode(fragment);

                        // select new content
                        if (selectContent) {
                            if (firstInsertedNode) {
                                range.setStartBefore(firstInsertedNode);
                                range.setEndAfter(lastInsertedNode);
                            }

                            // remove already existing ranges
                            selection.removeAllRanges();
                            // add new selection
                            selection.addRange(range);
                        }
                    }
                },

                /**
                 * Register a command by name ('colorPicker') to open a color picker dialog.
                 * This function also selects an existing *color-span* and handels other
                 * possibilities of selections.
                 *
                 * @param {scribe} object - reference to scribe editor
                 * @param {commandName} string - name of the plugin command
                 */
                registerCommand = function (scribe, commandName) {
                    // register new scribe command
                    var command = new scribe.api.Command(commandName);

                    // function which is executed if the plugin button was pressed
                    command.execute = function() {
                        var selection = new scribe.api.Selection(),
                            node = selectSpanContent(selection),
                            currentColor,
                            // current callback function for the color picker widget
                            // => this is necessary because of the scoped variables (node, selection)
                            callback = function(color) {
                                var content,
                                    replaceSelectionHtml;

                                // get selection and add range again to be able to remove colors again
                                getSelection().removeAllRanges();
                                getSelection().addRange(selection.range);

                                // get existing content
                                content = node ? node.innerHTML : selection.range.toString();
                                // create new html fragment
                                replaceSelectionHtml = color === null ? content : "<span style=\"color: " + color + ";\">" + content + "</span>" ;
                                // insert new html fragment
                                replaceSelectionWithHtml(scribe, replaceSelectionHtml, false);

                                scribe.el.focus();
                                getSelection().collapseToEnd();
                            };

                        // if a node was selected
                        if (node) {
                            selection = new scribe.api.Selection();
                            currentColor = node.style.color;
                        }

                        // handle clitches with non selected content (nothing is selected)
                        if (selection.selection.isCollapsed) {
                            window.setTimeout(function() {
                                scribe.el.focus();
                                selection.selection.collapseToEnd();
                            }, 0);
                            return;
                        }

                        require([
                            "ep/ui/colorpicker"
                        ], function(ep) {
                            if (!epColorPicker) {
                                epColorPicker = ep(colorPicker);
                            }

                            if (epColorPicker.data('uiUiColorpicker') !== undefined) {
                                // update callback function
                                epColorPicker.uiColorpicker('option', 'callback', callback);
                                // set current color
                                currentColor && epColorPicker.uiColorpicker('option', 'color', currentColor);
                                // open color picker dialog
                                epColorPicker.uiColorpicker('open');
                            } else {
                                // initialize color picker widget
                                epColorPicker.uiColorpicker({
                                    color: currentColor ? currentColor : 'black',
                                    autoOpen: true,
                                    orientation: 'right',
                                    layout: 'scribe',
                                    callback: callback
                                });
                            }
                        });
                    };

                    // enables/disables color picker icon in toolbar
                    command.queryEnabled = function () {
                        var selection = new scribe.api.Selection();

                        return !!selection.getContaining(function(node) {
                            // node is either a DOM or a text node
                            var parentNode = node.parentNode;

                            return (
                                (parentNode && parentNode.nodeName.toUpperCase() === nodeName && stylePattern.test(parentNode.style.cssText))
                                ||
                                ((selection.range.commonAncestorContainer.nodeType === 3) && selection.range.toString().length > 0)
                            );
                        });
                    };

                    return command;
                };

            // create command object for color picker
            var command = {
                command: registerCommand(scribe, colorPicker.dataset.commandName, colorPicker.dataset.colorpickerValue),
                title: colorPicker.textContent,
                name: colorPicker.dataset.commandName
            };

            // register command
            scribe.commands[command.name] = command.command;
        };
    };

    // Module system magic dance
    if (typeof module !== 'undefined' && typeof module.exports === 'object') {
        module.exports = scribePluginColorpicker;
    } else if (typeof define === 'function' && typeof define.amd === 'object') {
        define(function () {
            return scribePluginColorpicker;
        });
    } else {
        window.scribePluginColorpicker = scribePluginColorpicker;
    }
}());